<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Config;

use ConnectPay\Payment\Model\Request\Signature;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Filesystem\DirectoryList;
use Magento\Framework\Filesystem\DriverInterface;
use Magento\Framework\Math\Random;
use Magento\Framework\UrlInterface;
use Magento\Payment\Gateway\Config\Config as GatewayConfig;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

class Config extends GatewayConfig implements ConfigInterface
{
    /**
     * @var Random
     */
    private $random;

    /**
     * @var DirectoryList
     */
    private $directory;

    /**
     * @var DriverInterface
     */
    private $driver;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var Signature
     */
    private $signature;

    /**
     * @param Random $random
     * @param DirectoryList $directory
     * @param DriverInterface $driver
     * @param LoggerInterface $logger
     * @param StoreManagerInterface $storeManager
     * @param Signature $signature
     * @param ScopeConfigInterface $scopeConfig
     * @param string|null $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        Random                $random,
        DirectoryList         $directory,
        DriverInterface       $driver,
        LoggerInterface       $logger,
        StoreManagerInterface $storeManager,
        Signature             $signature,
        ScopeConfigInterface  $scopeConfig,
        string                $methodCode = null,
        string                $pathPattern = GatewayConfig::DEFAULT_PATH_PATTERN
    )
    {
        $this->random = $random;
        $this->directory = $directory;
        $this->driver = $driver;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        $this->signature = $signature;
        parent::__construct($scopeConfig, $methodCode, $pathPattern);
    }

    public function getBrandId(): string
    {
        return (string)$this->getValue(self::XML_PATH_BRAND_ID);
    }

    public function getWidgetLanguage(): string
    {
        return (string)$this->getValue(self::XML_PATH_WIDGET_LANGUAGE);
    }

    public function getProviderCountryCode(): string
    {
        return (string)$this->getValue(self::XML_PATH_PROVIDER_COUNTRY_CODE);
    }

    public function getProviderId(): array
    {
        $providerString = $this->getValue(self::XML_PATH_PROVIDER_ID);
        if (is_null($providerString)) {
            return [];
        }
        return explode(',', $providerString);
    }

    public function getClientId(): string
    {
        return (string)$this->getValue(self::XML_PATH_CLIENT_ID);
    }

    public function getClientSecret(): string
    {
        return (string)$this->getValue(self::XML_PATH_CLIENT_SECRET);
    }

    public function getPrivateKey(): string
    {
        $filename = (string)$this->getValue(self::XML_PATH_PRIVATE_KEY);
        return $this->getPath($filename);
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getRedirectUrl(): string
    {
        $storeUrl = $this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB, true);
        return $storeUrl . 'checkout/onepage/success';
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getNotificationUrl(): string
    {
        $storeUrl = $this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB, true);
        return $storeUrl . Config::NOTIFICATION_URL;
    }

    /**
     * @return string
     * @throws LocalizedException
     */
    public function getUuid(): string
    {
        return $this->random->getUniqueHash();
    }

    /**
     * @return bool
     */
    public function isTestModeEnabled(): bool
    {
        return (bool)$this->getValue(self::XML_PATH_SANDBOX_MODE);
    }

    /**
     * @param string $filename
     * @return string
     */
    protected function getPath(string $filename): string
    {
        try {
            $path = $this->directory->getPath('var') . DIRECTORY_SEPARATOR . self::CONNECTPAY_DIR . DIRECTORY_SEPARATOR . $filename;
            if (!$this->driver->isExists($path)) {
                $path = '';
            }
        } catch (FileSystemException $exception) {
            $this->logger->error($exception);
            $path = '';
        }

        return $path;
    }

    /**
     * @param $data
     * @param $http_response_header
     * @param $url
     * @param $endpoint
     * @return string
     */
    public function getSignature($data, $http_response_header, $url, $endpoint): string
    {
        $privateKey = $this->getPrivateKey();
        $file = file_get_contents($privateKey);
        $pkeyid = openssl_get_privatekey($file);
        return $this->signature->getSignature($data, $http_response_header, $url, $endpoint, $pkeyid);
    }
}
